/*
    SherryParameter.h
  
    Copyright (C) 1998, 1999 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef SherryParameter_H
#define SherryParameter_H

#include <cassert>

#include "util/mytype.h"

using namespace std;

class SherryParameter
{
public:
  struct ScreenParameter
  {
    uint16 screenNumber;
    uint16 sizeX;
    uint16 sizeY;
    uint8 transparentColor;
  };

  struct PaletteParameter
  {
    uint8 paletteNumber;
    uint8 red;
    uint8 green;
    uint8 blue;
  };

  struct PaletteNumberParameter
  {
    uint8 fromPalette;
    uint8 toPalette;
  };
  
  enum ParameterType
  {
    CHAR,
    UINT8,
    UINT16,
    UINT32,

    CHAR_ARRAY,
    UINT8_ARRAY,
    UINT16_ARRAY,

    //uint16,uint16,uint16,uint8array
    //ScreenParameterarray
    //for 0x25
    SCREEN_ARRAY,
  
    //uint8,uint8,uint8,uint8array
    //PaletteParameterarray
    //for 0x41
    PALETTE_ARRAY,

    //uint8[32]
    //for pallete copy mask
    PALETTE_COPY_MASK,

    //uint16 uint16array
    //PalettePariParameterarray
    //for partial palette copy
    PALETTE_NUMBER_ARRAY,
    
    ////////////////////////////////////////////////////////////inner use
    //parameterϤǽꡣ
    END_OF_PARAMETER,
    INVALID,
  };

  
  union Value
  {
    char charParameter;
    uint8 uint8Parameter;
    uint16 uint16Parameter;
    uint32 uint32Parameter;

    char* charArray;
    uint8* uint8Array;
    uint16* uint16Array;
    
    ScreenParameter* screenArray;
    PaletteParameter* paletteArray;
    PaletteNumberParameter* paletteNumberArray;
  };

private:
  const ParameterType parameterType;
  Value value;
  const int sizeOfArray;//valuearray typeλarray礭
  
public:
  SherryParameter (char charParameter);
  SherryParameter (uint8 uint8Parameter);
  SherryParameter (uint16 uint16Parameter);
  SherryParameter (uint32 uint32Parameter);
  
  SherryParameter (const char* charArray, int size);
  SherryParameter (const uint8* uint8Array, int size);
  SherryParameter (const uint16* uint16Array, int size);

  SherryParameter (const ScreenParameter* screenArray, int size);
  SherryParameter (const PaletteParameter* paletteArray, int size);
  SherryParameter (const uint8 paletteCopyMask[32]);
  SherryParameter (const PaletteNumberParameter* paletteNumberArray, int size);
  
  ~SherryParameter ();
  

  /////////////////////////////////////////////////////////access method
  int getArraySize () const
    {
      assert (parameterType == CHAR_ARRAY ||
	      parameterType == UINT8_ARRAY ||
	      parameterType == UINT16_ARRAY ||
	      parameterType == SCREEN_ARRAY ||
	      parameterType == PALETTE_ARRAY ||
	      parameterType == PALETTE_COPY_MASK ||
      	      parameterType == PALETTE_NUMBER_ARRAY);
      return sizeOfArray;
    };
  
  char getChar () const
    {
      assert (parameterType == CHAR);
      return value.charParameter;
    };
  uint8 getUint8 () const
    {
      assert (parameterType == UINT8);
      return value.uint8Parameter;
    };
  uint16 getUint16 () const
    {
      assert (parameterType == UINT16);
      return value.uint16Parameter;
    };
  uint32 getUint32 () const
    {
      assert (parameterType == UINT32);
      return value.uint32Parameter;
    };

  const char* getCharArray () const 
    {
      assert (parameterType == CHAR_ARRAY);
      return value.charArray;
    };
  const uint8* getUint8Array () const 
    {
      assert (parameterType == UINT8_ARRAY);
      return value.uint8Array;
    };
  const uint16* getUint16Array () const 
    {
      assert (parameterType == UINT16_ARRAY);
      return value.uint16Array;
    };
  const ScreenParameter*  getScreenArray () const 
    {
      assert (parameterType == SCREEN_ARRAY);
      return value.screenArray;
    };
  const PaletteParameter* getPaletteArray () const 
    {
      assert (parameterType == PALETTE_ARRAY);
      return value.paletteArray;
    };
  const uint8* getPaletteCopyArray () const 
    {
      assert (parameterType == PALETTE_NUMBER_ARRAY);
      assert (sizeOfArray == 32);
      return value.uint8Array;
    };
  const PaletteNumberParameter* getPaletteNumberArray () const 
    {
      assert (parameterType == PALETTE_NUMBER_ARRAY);
      return value.paletteNumberArray;
    };
};

#endif // SherryParameter_H
