/*
    SherryWrdFile.h
  
    Copyright (C) 1998, 1999 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef SherryWrdFile_H
#define SherryWrdFile_H

#include <list>

#include "util/mytype.h"
#include "SourceFileItem.h"
#include "CompilerPrivateInfoItem.h"
#include "SherryCommand.h"
#include "SherryInputStream.h"

using namespace std;

//sherry wrd fileɤ߹ǡcommandlinked list롣
class SherryWrdFile
{
public:
  static const char SherryMagic[12] = "Sherry WRD\0";
  typedef list<const SherryCommand*> CommandsList;
  typedef CommandsList::const_iterator Command;
  
private:
  string filename;
  int32 version;//sherry WRD version
  static const int16 TIME_BASE_IS_NOT_SET = -1;
  int16 timeBase;//time base:4ʬstep

  list<const string> compilerName;
  list<SourceFileItem> sourceFiles;
  typedef list<const CompilerPrivateInfoItem*> CompilerPrivateInfo ;
  CompilerPrivateInfo compilerPrivateInfo;
  CommandsList commands;
  
  //fileload
  void load (const string& filename__);
  //data blockɤ߹
  void readDataBlock (SherryInputStream& ifs);

public:
  SherryWrdFile (const string& filename__);
  ~SherryWrdFile ();

  //acces method
  const string& getFileName () const {return filename;};
  int32 getVersion () const {return version;};
  int16 getTimeBase () const {return timeBase;};
  const list<const string>& getCompilerName () const {return compilerName;};
  const list<SourceFileItem>& getSourceFiles () const {return sourceFiles;};
  const CompilerPrivateInfo& getCompilerPrivateInfo () const
    {return compilerPrivateInfo;};
  const CommandsList& getCommandsList () const {return commands;};

  
public:
  //exceptional class
  class SError {};
  class FileIsNotFound:public SError {};
  class FileIsTooShort:public SError {};
  class FileIsNotSherryWrd:public SError {};
  class UnsupportedVersion:public SError {};
  class TimeBaseIsNotSet:public SError {};
};
#endif //SherryWrdFile_H
