/*
    GTKBase.cc
  
    Copyright (C) 1998, 1999 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <gtk/gtk.h>

#include "util/onceUtil.h"
#include "GTKBase.h"

#ifdef LOCALLY_SCHEDULED_THREAD
# include "util/schedYield.h"
#endif

Mutex GTKBase::gtkLock;
int GTKBase::gtkArgc = 1;
char** GTKBase::gtkArgv = NULL;
int GTKBase::nrChild = 0;
pthread_once_t GTKBase::gtkInit = PTHREAD_ONCE_INIT;
pthread_t GTKBase::gtkMainThread;

void GTKBase::initGTK ()
{
  g_thread_init (NULL);
  if (!g_thread_supported ())
    {
      cerr << "Glib does not support multi thread. aborting" << endl;
      exit (1);//XXXX
    }
  
#if 0
  gtkArgc = 1;
  gtkArgv = new char*[gtkArgc+1];
#define MAX_ARGV_LENGTH 16
  for (int i = 0; i < gtkArgc+1; i++)
    gtkArgv[i] = new char[MAX_ARGV_LENGTH];
#undef MAX_ARGV_LENGTH
  gtk_init (&gtkArgc, &gtkArgv);//memory leak?
  for (int i = 0; i < gtkArgc+1; i++)
    delete[] gtkArgv[i];
  delete[] gtkArgv;
#else
  gtk_init (0, NULL);
#endif
}


void* GTKBase::gtkMain (void*)
{
#ifdef LOCALLY_SCHEDULED_THREAD
  //???$B$3$l$G$&$^$/9T$/(B?
  gdk_threads_enter ();
  while (!gtk_main_iteration_do (FALSE))
    {
      gdk_threads_leave ();
      schedYield ();
      gdk_threads_enter ();
    }
  gdk_threads_leave ();
  return NULL;
#else // LOCALLY_SCHEDLUED_THREAD
  gdk_threads_enter ();
  gtk_main ();
  gdk_threads_leave ();
  return NULL;
#endif  // LOCALLY_SCHEDLUED_THREAD
}


GTKBase::GTKBase ()
{
  gtkLock.lock ();
  initOnce (&gtkInit, initGTK);
  if (nrChild == 0)
    pthread_create (&gtkMainThread,NULL, &GTKBase::gtkMain, NULL);
  nrChild++;
  gtkLock.unlock ();
}


GTKBase::~GTKBase ()
{
  gtkLock.lock ();
  nrChild--;
  if (nrChild == 0)
    {
      gdk_threads_enter ();
      gtk_main_quit ();
      //gdk_threads_wake ();//needs this???
      gdk_threads_leave ();
      pthread_join (gtkMainThread, NULL);
    }
  gtkLock.unlock ();
}
