/*
    findVisual.cc
  
    Copyright (C) 1998, 1999 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <vector>
#include <algorithm>
#include <functional>
#include <iostream>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "findVisual.h"
#include "util/mytype.h"

using namespace std;

void findVisual (Display* display, int screenNumber,
		 Visual** visual, int* visualClass, int* depth)
{
  int countReturn;
  int* depths = XListDepths (display,  screenNumber, &countReturn);
  vector<int> depthList;
  for (int i = 0; i<countReturn; i++)
    {
      if (depths[i] >= 8)//8bpp$B0J>e$N$_;D$9!#(B
	depthList.push_back (depths[i]);
    }
  XFree (depths);
  
  if (depthList.size () == 0)
    {
      cerr << "ERROR:findVisual"
	   << " display does not support needed depth>=8" << endl;
      //throw IError ("This display does not support needed depths>=8.");
      //XXX$B$b$C$H$^$7$J%(%i!<=hM}$,$"$kH&!#(B
      exit (1);
    }

  sort (depthList.begin (), depthList.end (), less<int>());//$B>:=g$KJB$Y$k!#(B
  //depth$B$GM-$jF@$k$N$O(B8bpp,16bpp,24bpp,32bpp$B$+(B???
  //PseudoColor,DirectColor$B$N=g$GC5$9!#(B
  const int visualClasses[] = {PseudoColor, DirectColor};
  const int nrVisualClass = sizeof (visualClass)/sizeof (int);
  XVisualInfo visualInfo;
  int i;
  for (i = 0; i < nrVisualClass; i++)
    {
      for (vector<int>::const_iterator j = depthList.begin ();
	   j != depthList.end ();
	   j++)
	{
	  if (XMatchVisualInfo (display, screenNumber,
				*j, visualClasses[i], &visualInfo))
	    goto VISUAL_FOUND;//$BL\E*$N(Bvisual$B$,8+$D$+$C$?!#(B
	}
      if (i == nrVisualClass-1)
	{//$B;H$($k(BVisual$B$,8+$D$+$i$J$+$C$?!#(B
	  cerr << "findVisual"
	       << " can not found usable visual type." << endl;
	  //throw IError ("Error:IOXWrd::initlaise"
	  //	" can not found usable visual typ.");
	  //XXX$B$b$C$H$^$7$J%(%i!<=hM}$,$"$kH&!#(B
	  exit (1);
	}
    }
  

 VISUAL_FOUND:
  *visual = visualInfo.visual;
  *depth = visualInfo.depth;
  *visualClass = visualInfo.c_class;
  assert (screenNumber == visualInfo.screen);
}
