/*
    SerialDevice.h
  
    Copyright (C) 1998, 1999 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef SerialDevice_H
#define SerialDevice_H

#include <termios.h>

#include <string>

#include "util/mytype.h"

using namespace std;

class SerialDevice
{
public:
  static const char* const DEFAULT_SERIAL_DEVICE;
  static const int DEFAULT_BPS;
  
private:
  static const speed_t B31250 = B50;
  int serialfd;
  void open (const string& device, int bps);
  
public:
  //bps$B$O(B35400,31250$B$N$_(Bsupport
  SerialDevice ();
  SerialDevice (const string& device);
  SerialDevice (const string& device, int bps);
  ~SerialDevice ();
  
  void write (const uchar* data, ssize_t length);
  
  //exeptional classes
  class Error {};
  class BogusBPS:public Error {};
  class CanNotOpenDevice:public Error {};
  class CanNotGetDeviceAttribute:public Error {};
  class CanNotSetDeviceAttribute:public Error {};
};

#endif // SerialDevice_H
