/*
    newSequencer.cc
  
    Copyright (C) 1998, 1999 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "Sequencer.h"
#include "InfoSequencer.h"

#if defined (HAVE_SYS_SOUNDCARD_H) || defined (HAVE_MACHINE_SOUNDCARD_H)
# include "Sequencer/OSS/MIDI/SequencerMIDI.h"
#else
# include "Sequencer/SequencerNull.h"
#endif

Sequencer* Sequencer::newSequencer (DlHandler** dlHandler_, const string& name,
				    State* state_, Option* option_)
{
  assert (state_ != NULL);
  assert (option_ != NULL);
  assert (dlHandler_ != NULL);
  
  *dlHandler_ = NULL;
  if (name.length () == 0)
    goto CREATE_DEFAULT_SEQUENCER;

  *dlHandler_ = DlHandler::newOpenDlHandler ("Sequencer",
					     name, option_);

  if (*dlHandler_ == NULL)
    goto CREATE_DEFAULT_SEQUENCER;

  Sequencer* ret;
  try
    {
      ret = InfoSequencer::createSequencer (*dlHandler_, name,
					    state_, option_);
    }
  catch (Sequencer::SequencerError&)
    {
      delete *dlHandler_;
      *dlHandler_ = NULL;
      goto CREATE_DEFAULT_SEQUENCER;
    }

  if (ret == NULL)
    {
      delete *dlHandler_;
      *dlHandler_ = NULL;
      goto CREATE_DEFAULT_SEQUENCER;
    }
  
  return ret;

 CREATE_DEFAULT_SEQUENCER:
#if defined (HAVE_SYS_SOUNDCARD_H) || defined (HAVE_MACHINE_SOUNDCARD_H)
  return new SequencerMIDI (state_, option_->midiDeviceNum, option_);
#else
  return new SequencerNull (state_, option_);
#endif
}
