/*
    SongFactory.cc
  
    Copyright (C) 1998, 1999 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <string>

#include "SongFactory.h"

using namespace std;

SongFactory SongFactory::songFactory;
Mutex SongFactory::lock;

SongFactory::SongFactory ()
{
  //do nothing
}

SongFactory::~SongFactory ()
{
  for (list<DlHandler*>::iterator i = dlHandlerList.begin ();
       i != dlHandlerList.end ();
       i++)
    delete *i;
}


void SongFactory::__addSongModule (const string& songModuleName,
				   Option* option__)
{
  SongFactory::MonitorLock monitorLock ();
  
  DlHandler* dlHandler = new DlHandler (option__);
  if (dlHandler->openLibSo ("Song", songModuleName) != NULL)
    {
      string songInfoName = "InfoSong";
      songInfoName += songModuleName;
      void* infoSong = dlHandler->sym (songInfoName);
      if (infoSong != NULL)
	{
	  infoSongList.push_front (static_cast<InfoSong*>(infoSong));
	}
      else
	{//fail to sym
	  cerr << "SongFactory::addSongModule can not sym "
	       << songInfoName << endl;
	  delete dlHandler;
	}
    }
  else
    {//fail to open dynamic library
      cerr << "SongFactory::addSongModule can not open "
	   << "libSong" << songModuleName << ".so" << endl;
      delete dlHandler;
    }
}
