/*
    Random.h
  
    Copyright (C) 1998, 1999 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef Random_H
#define Random_H

#include <cstdlib>
#include <sys/time.h>
#include <unistd.h>

#ifndef HAVE_RAND_R
# include "Mutex.h"
# include "onceUtil.h"
#endif

using namespace std;

class Random
{
private:
  unsigned int seed;

#ifndef HAVE_RAND_R
  static Mutex mutex;
  class Monitor
  {
  public:
    Monitor () {mutex.lock ();};
    ~Monitor () {mutex.unlock ();};
  };
  friend Monitor;

  static pthread_once_t once;
  static void srand ();
#endif
  
public:
  Random ()
    {
#ifdef HAVE_RAND_R
      //???$B$3$s$J;v$r$7$F$bBg>fIW$J$N(B?
      timeval timeVal;
      gettimeofday (&timeVal, NULL);
      seed = timeVal.tv_usec;
#else
      Monitor monitor;
      initOnce (&once, &Random::srand);
#endif
    };
  Random (unsigned int seed__):seed (seed__)
    {
#ifndef HAVE_RAND_R
      Monitor monitor;
      initOnce (&once, &Random::srand);
#endif
    };
  ~Random () {};

  int operator () ()
    {
      //extern int rand_r __P ((unsigned int *__seed));
      /* Return non-negative, long integer in [0,2^31).  */
      //extern int lrand48_r __P ((struct drand48_data *__buffer, long *__result));
#ifdef HAVE_RAND_R
      return rand_r (&seed);
#else
      Monitor monitor;
      return rand ();
#endif
    };

  //return [0,upper)
  int operator () (int upper)
    {
#ifdef HAVE_RAND_R
#if 0
      const int base = rand_r (&seed);
      double ret = (upper+1.0) *  base /(RAND_MAX+1.0);
      cout << "base=" << base << ":Random=" << ret << endl;
      return static_cast<int> (ret);
#endif
      //rand_r$B$OIi$NCM$rJV$9$3$H$,$"$k(B?
      //[0,RAND_MAX]$B$rJV$9$H;W$C$F$$$?$N$@$1$l$I!#(B
      return static_cast<int> ((upper+1.0) * abs (rand_r (&seed))/(RAND_MAX+1.0));
      //return static_cast<int> ((upper+1.0) * rand_r (&seed)/(RAND_MAX+1.0));
#else
      Monitor monitor;
      return static_cast<int> ((upper+1.0) * rand ()/(RAND_MAX+1.0));
#endif
    };
};
#endif Random_H
